/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */

package com.supinfo.spitfireboard.services;

import com.supinfo.spitfireboard.dao.BoardDao;
import com.supinfo.spitfireboard.dao.CategoryDao;
import com.supinfo.spitfireboard.dao.PostDao;
import com.supinfo.spitfireboard.dao.TopicDao;
import com.supinfo.spitfireboard.entities.Board;
import com.supinfo.spitfireboard.entities.Category;
import com.supinfo.spitfireboard.entities.Post;
import com.supinfo.spitfireboard.entities.Topic;
import java.util.Date;
import java.util.List;
import javax.ejb.EJB;
import javax.ejb.Stateless;

/**
 *
 * @author Fabien LOISON
 */
@Stateless
public class ForumService {
    
    @EJB
    private CategoryDao categoryDao;
    
    @EJB
    private BoardDao boardDao;
    
    @EJB
    private TopicDao topicDao;
    
    @EJB
    private PostDao postDao;

    public List<Category> getCategories() {
        return categoryDao.getAll();
    }
    
    public Board getBoard(Long id) {
        return boardDao.getById(id);
    }
    
    public List<Topic> getBoardTopics(Long boardId) {
        return topicDao.getTopicsByBoardId(boardId);
    }
    
    public Topic getTopic(Long id) {
        return topicDao.getById(id);
    }
    
    public List<Post> getTopicPosts(Long topicId) {
        return postDao.getPostsByTopicId(topicId);
    }
    
    public Long newPost(String text, long topicId, long userId) {
        Post post = new Post();
        
        post.setCreationDate(new Date());
        post.setMessage(text);
        post.setTopicId(topicId);
        post.setUserId(userId);
        
        return postDao.persist(post);
    }
    
    public void lockTopic(Long topicId) {
        Topic topic = topicDao.getById(topicId);
        topic.setLocked(true);
        topicDao.persistTopic(topic);
    }
    
    public void unlockTopic(Long topicId) {
        Topic topic = topicDao.getById(topicId);
        topic.setLocked(false);
        topicDao.persistTopic(topic);
    }
    
    public Long newTopic(Long userId, Long boardId, String title, String message) {
        Topic topic = new Topic();
        
        topic.setName(title);
        topic.setBoardId(boardId);
        topic.setLocked(false);
        topic.setCreationDate(new Date());
        
        Long topicId = topicDao.persistTopic(topic);
        
        Post post = new Post();
        
        post.setMessage(message);
        post.setTopicId(topicId);
        post.setUserId(userId);
        post.setCreationDate(new Date());
        
        postDao.persist(post);
        
        return topicId;
    }
    
    public void deleteTopic(Long id) {
        topicDao.delete(id);
    }
    
    public void topicEdit(Long id, String title) {
        Topic topic = topicDao.getById(id);
        topic.setName(title);
        topicDao.persistTopic(topic);
    }
    
    public void topicMove(Long topicId, Long boardId) {
        Topic topic = topicDao.getById(topicId);
        topic.setBoardId(boardId);
        topicDao.persistTopic(topic);
    }
    
    public void topicMerge(Long initialTopicId, Long mergeTopicId) {
        Topic topic = topicDao.getById(initialTopicId);
        List<Post> posts = postDao.getPostsByTopicId(initialTopicId);
        for (int i = 0; i < posts.size(); i++) {
            Long postId = posts.get(i).getId();
            Post post = postDao.getById(postId);
            post.setTopicId(mergeTopicId);
            
            postDao.persist(post);
        }
        
        this.deleteTopic(initialTopicId);
    }
    
    public Post getPost(Long id) {
        return postDao.getById(id);
    }
    
    public void deletePost(Long id) {
        postDao.delete(id);
    }
    
    public void postEdit(Long id, String message) {
        Post post = postDao.getById(id);
        post.setMessage(message);
        postDao.persist(post);
    }
}
